/*
 * Cmd_moteur.c
 *
 *  Created on: 20 nov. 2018
 *      Author: admin_aboyer
 *
 *  Fonctions associes au contrle moteur, bas sur une commande U/f.
 *  Les fonctions mathmatiques sont issues de la SDK AMMCLIB. Le type FLT est utilis.
 *  Dans le fichier SWLIBS_Config.h, donnez  SWLIBS_DEFAULT_IMPLEMENTATION la valeur
 *  SWLIBS_DEFAULT_IMPLEMENTATION_FLT, sinon erreur de build !
 */


#include "Cmd_moteur.h"
#include "gd3000.h"
#include "ConfigPIT.h"
#include "ClockConfig.h"
#include "SWLIBS_Typedefs.h"
#include "gflib.h"
#include "gmclib.h"
#include "mlib.h"
#include "pga411.h"
#include "ConfigGPIO.h"
#include "ConfigADC.h"
#include "ConfigPWM.h"
#include "Current_Sensor.h"
#include "User_Interface.h"
#include "modele_moteur_virtuel.h"


/*************************************************************************
 *variables de cmd_moteur.c :
 *************************************************************************/

//variables lies  la squence de pilotage moteur
//la variable Mode_Sequence_Moteur indique si on se trouve en mode de fonctionnement normal ou de calibration.
//en mode de calibration, on fait des acquisitions de courant de IphA et IphB pour compenser l'offset. LA boucle
//slow loop n'est pas effectue. En mode normal, les boucles fast loop et slow loop sont effectues normalement.
uint8_t Mode_Sequence_Moteur;

//variables lies au calcul de la commande de vitesse (rampe de vitesse)
extern tFloat Consigne_Vitesse; //la valeur finale de vitesse  atteindre
tFloat D_Vitesse; //la pente de la rampe --> changement de vitesse max pendant une priode
		//de boucle slow loop (exprim en RPM)
tFloat Commande_Vitesse; //la commande de vitesse en cours

//variables lies au contrle de la boucle slow loop
uint32_t cptr_slow_loop; //compteur qui permet de dterminer s'il faut excuter la boucle slow loop lors
	//d'une excution de la fast loop.
uint32_t nb_exec_FL_max_Exec_SL; //dtermine le nb max de boucle Fast Loop (FL)  excuter pour excuter
	//la boucle slow loop (SL)

//les variables lies  la commande des 3 phases du moteur (tensions dans les repres abc et dq, et rapports cycliques)
//les commandes dans les repres DQ, ABC et traduction en rapports cycliques.
SWLIBS_2Syst_FLT Udq_Commande; //commande moteur dans le repre dq, exprime en V
SWLIBS_3Syst_FLT Uabc_Commande; //commande moteur dans le repre abc, exprime en V (uniquement pour le test, pour afficher
//sous Freemaster les tensions de commande en V).

//Variables et paramtres lies  la commande PWM et son rglage
uint16_t Cmd_DutyA, Cmd_DutyB, Cmd_DutyC; //rapports cycliques des commandes des 3 phases,  fournir directement
							//au FlexPWM (exprims en top d'horloge)
extern uint32_t PWMPeriod;  //en nb de clock submodule
extern uint16_t DutyDefaultGD3000; //rapport cyclique par dfaut (en nb de clock submodule)
extern uint32_t DeadTime_PWM; ////valeur du deadtime (rgl au niveau du module FlexPWM), donn en nb de priodes
		//d'horloge MOTC (frq = 40 MHz --> priode = 25 ns). Mme valeur pour fronts montants et descendants
extern uint8_t Align_PWM; //rglage de l'alignement de la PWM : CENTER_ALIGN ou EDGE_ALIGN


//variables lies  la mesure des courants de phase
extern SWLIBS_3Syst_FLT Iphase; //courant dans les 3 phases (en A)
extern SWLIBS_3Syst_FLT Offset_Mes_Iph_mV;  //ventuel offset affectant la mesure du courant de phase (en mV)
extern SWLIBS_3Syst_FLT Offset_Mes_Iph_uV;
extern uint8_t Cptr_Mes_Cal;  //compteur pour la compensation de l'offset de mesure du courant
extern uint8_t Cal_Current_Done; //indique si la calibration de l'offset de la mesure du courant est achev
uint32_t Delay_Trig_ADC_us; //dlai en s entre le trigger issu de la PWM (dpassement VAL1 pour dclencher l'acquisition
// partir du moment o les 3 phases sont  0) et la commande d'chantillonnage envoye par le CTU  l'ADC.


//variables lies  la mesure de l'angle du rotor par le PGA411, ainsi qu' l'extraction de la vitesse rotor
extern uint8_t Resolution_PGA411; //rsolution du PG411 pour la mesure de l'angle du rotor
tFloat angle_rotor_elec; //en degr
tFloat angle_rotor_meca; //en degr --> ce qui est mesur par le rsolveur
tFloat velocity_rotor_elec; //en RPM
tFloat velocity_rotor_meca; //en RPM
uint32_t nb_exec_loop_max_speed;  //nombre d'acquisition de donnes de position rotor pour estimer la vitesse
uint8_t cptr_speed;  //pour compter le nb de donnes de position rotor que l'on a acquise.
//pour l'valuation de la vitesse  partir des mesures d'angle rotor (par l'interface rsolveur) :
tFloat tps_integration_vitesse;  //le temps pendant lequel on fait le calcul d'intgration de l'angle rotor parcouru pendant
	//la boucle d'valuation de la vitesse.
tFloat increment_angle_rotor;  //suivi de l'angle parcouru pendant la boucle d'integration de la vitesse.
tFloat angle_rotor_meca_precedent; //valeur prcdemment mesure de l'angle (pour reprer un franchissement de 360)
tFloat cte_calcul_RPM_from_deg_us;
tFloat cte_calcul_RPM_from_rad_us;


//variables lies  l'estimation de l'angle lectrique du rotor, requise pour la commande U/f.
//La commande U/f tant en boucle ouverte, on n'utilise pas la mesure de l'angle rotor pour la commande. Sinon,
//risque d'instabilit.
tFloat Angle_Rotor_Estime; //l'angle du rotor, estim  partir de la commande de vitesse.
	//Cette variable est exprim en radian, compris entre -pi et +pi.
	//attention : il s'agit d'un angle lectrique ! Il est dduit de Commande_Vitesse, qui est bien la
	//vitesse mcanique. Mais le nb de paires de poles est pris en compte dans le calcul de l'angle rotor.
tFloat const_calcul_RPM_Radian; //constante permettant l'estimation de l'angle rotor  partir de la vitesse.
			//Angle = Vitesse_RPM*pi/30*nb_paire_poles*T_FAST_LOOP


uint8_t Mode_Lecture_Courant_Angle; //= 0 --> angle rotor et courants de phase obtenus par mesure --> pour les essais sur
									//moteur.
									//=1 --> angle rotor et courants de phase obtenus par calcul (utilisation d'un modle
									//de moteur virtuel, donc ne convient pas pour des essais rels).

//indique si on utilise la Space Vector Modulation (SVM)
uint8_t SVM_Activate;

/*****************************************************************************/
/* init_Cmd_Moteur_Variables                                                  */
/* Description: initialisation,  appeler au premier dmarrage, de tous les paramtres et signaux d'entre
 * de la commande du moteur, ainsi que des paramtres mesurs (courants, angle et vitesse rotor).
 * La variable Moteur_Reel_Ou_Virtuel = MOTEUR_REEL ou MOTEUR_VIRTUEL, pour indiquer si on pilote rellement le moteur
 * avec l'utilisation de la vraie mesure de courant et d'angle, ou si on se base sur un modle virtuel (pas de pilotage
 * d'un vrai moteur, pas de capteur, ddi uniquement  la validation du logiciel).
 * La variable DeadTime donne la valeur du deadtime, dfinie par :
 *  DeadTime*Freq_MOTC (40 MHz). Mme valeur sur fronts montants et descendants
 * La variable Align_PWM rgle l'alignement de la PWM : deux modes d'alignement sont supports :
 * EDGE_ALIGN et CENTER_ALIGN.
 */
/*****************************************************************************/
void init_Cmd_Moteur_Variables(uint8_t Moteur_Reel_Ou_Virtuel, uint8_t SVM_On_or_Off, uint32_t DeadTime, uint32_t Align) {
	D_Vitesse = MLIB_Mul_FLT(ACCELERATION_RAMPE_VITESSE,T_SLOW_LOOP_MS);
	nb_exec_FL_max_Exec_SL = T_SLOW_LOOP_MS*1000/T_FAST_LOOP_US;
	const_calcul_RPM_Radian = (tFloat) (0.1047*PAIRE_POLE*T_FAST_LOOP_US);
	Mode_Sequence_Moteur = MODE_FCT_CAL;

	//paramtres pour la PWM
	//initialisation de la commande PWM du GD3000
	PWMPeriod = FLEXPWM_CLK_MHz*T_FAST_LOOP_US;
	DutyDefaultGD3000 = PWMPeriod/2;
//	DeadTime_PWM  = 100; //valeur du deadtime = DeadTime_PWM*Freq_MOTC (40 MHz) ts et descendants
//	Align_PWM = CENTER_ALIGN; //EDGE_ALIGN ou CENTER_ALIGN
	DeadTime_PWM  = DeadTime;
	Align_PWM = Align;

	//paramtres pour la mesure de courant
	Delay_Trig_ADC_us = 0; //dlai entre trigger PWM et acquisition du courant low side par l'ADC (en s)

	//initialisation des paramtres de mesure de l'angle et de la vitesse
	tps_integration_vitesse = (nb_exec_FL_max_Exec_SL-1)*T_FAST_LOOP_US; //exprim en s
	cte_calcul_RPM_from_deg_us = (tFloat) (60000000/360);
	cte_calcul_RPM_from_rad_us = (tFloat) (60000000/TWO_PI_FLT);

	raz_Cmd_Moteur_Variables();

	//pour indiquer si on se base sur des mesures relles des courants de phase et de l'angle rotor --> essai sur moteur
	//rel, ou si on se base sur un modle de moteur virtuel.
	//Mode_Lecture_Courant_Angle = MOTEUR_REEL ou MOTEUR_VIRTUEL
	Mode_Lecture_Courant_Angle = Moteur_Reel_Ou_Virtuel;
	//initialisation des variables et paramtres du modle de moteur virtuel
	if (Mode_Lecture_Courant_Angle == MOTEUR_VIRTUEL) {
		init_modele_moteur_virtuel();
	}

	SVM_Activate = SVM_On_or_Off;
}

/*****************************************************************************/
/* raz_Cmd_Moteur_Reel_Variables                                                  */
/* Description: remise  zro des variables requises pour le contrle de type U/f
 * A utiliser uniquement aprs un arrt du moteur, jamais quand le moteur tourne.
 */
/*****************************************************************************/
void raz_Cmd_Moteur_Variables(void) {
	Consigne_Vitesse = 0;
	Commande_Vitesse = 0;

	Udq_Commande.fltArg1 = 0;
	Udq_Commande.fltArg2 = 0;
	Uabc_Commande.fltArg1 = 0;
	Uabc_Commande.fltArg2 = 0;
	Uabc_Commande.fltArg3 = 0;
	Iphase.fltArg1 = 0;
	Iphase.fltArg2 = 0;
	Iphase.fltArg2 = 0;
	cptr_slow_loop = 0;
	Angle_Rotor_Estime = 0;

	//ors du prochain dmarrage, le programme entrera en mode de calibration de l'offset des ADC
	Mode_Sequence_Moteur = MODE_FCT_CAL;
	Cptr_Mes_Cal = 0;

	//initialisation des paramtres de mesure de l'angle et de la vitesse
	increment_angle_rotor = 0;
	angle_rotor_meca_precedent = 0;
	angle_rotor_elec = 0;
	angle_rotor_meca = 0;
	velocity_rotor_elec = 0;
	velocity_rotor_meca = 0;
}


/*****************************************************************************/
/* raz_Entree_Cmd                                                  */
/* Description: remise  zro des commandes requises pour le contrle de type U/f
 * --> uniquement la commande de vitesse. On met  0 aussi les paramtres du modle du moteur
 * dans le cas o l'on est mode simulation.
 * A utiliser aprs un arrt du moteur.
 */
/*****************************************************************************/
void raz_Entree_Cmd(void) {
	//raz de la rampe de vitesse
	Commande_Vitesse = 0;
	
	//raz des paramtres du modle du moteur qui stockent les valeurs de la vitesse, du couple, de la tension
	//de commande, du courant de phase  l'itration Fast Loop prcdente.
	if (Mode_Lecture_Courant_Angle == MOTEUR_VIRTUEL) {
		raz_variables_moteur_virtuel();
	}
}


/*****************************************************************************/
/* Exec_Fast_Loop                                                  */
/* Description: cette fonction gre l'excution de la boucle dite Fast Loop.
 * Cette boucle calcule les tensions de phase (Ua,Ub,Uc)  partir des
 * consignes (Ud,Uq).
 * Les tensions Ud,Uq sont exprimes en V.
 * Elle renvoie les rapports cycliques au format entier 16 bits.                                              */
/*****************************************************************************/
SWLIBS_3Syst_F16 Exec_Fast_Loop(SWLIBS_2Syst_FLT Udq_ref) {
	SWLIBS_3Syst_F16 Uabc_Duty_F16;
	SWLIBS_2Syst_FLT CosSinOut, AlphaBetaOut,DQ_Out;
	SWLIBS_3Syst_FLT PWM_abc_Out;
	tU32 SVMSector;

	DQ_Out.fltArg1 = Udq_ref.fltArg1;
	DQ_Out.fltArg2 = Udq_ref.fltArg2;

	Angle_Rotor_Estime = Estimate_Rotor_Angle(Commande_Vitesse,Angle_Rotor_Estime);

	//calcul du cos et du sin de l'angle rotor, ncessaire au calcul de la transfo inverse de Park.
	//GFLIB_SINCOS_DEFAULT_FLT contient l'ensemble des coefs de de Taylor pour valuer les fonctions
	//cos et sin par un polynome quivalent. Voir p 440 (partie 5.33.3.5 doc AMMCLIB) pour un exemple
	//d'utilisation de cette fonction.
	GFLIB_SinCos_FLT(Angle_Rotor_Estime,&CosSinOut,GFLIB_SINCOS_DEFAULT_FLT);

	//fonction Park inverse pour exprimer les consignes de tension Ud,Uq dans le repre alpha_beta.
	GMCLIB_ParkInv_FLT(&AlphaBetaOut,&CosSinOut,&DQ_Out);

	//mapping des tensions Ualpha,Ubeta dans le repre abc en utilisant la standard space vector modulation
	//la sortie donne directement un rapport cyclique (valeur comprise entre 0 et 1,  condition que l'on ait
	//normalis les tensions d'entres par rapport  la tension de phase max)
	AlphaBetaOut.fltArg1 = MLIB_Div_FLT(AlphaBetaOut.fltArg1,UPHASE_MAX_V);
	AlphaBetaOut.fltArg2 = MLIB_Div_FLT(AlphaBetaOut.fltArg2,UPHASE_MAX_V);


	//mapping Ualpha et Ubeta --> rapports cycliques associs  Ua, Ub et Uc
	if (SVM_Activate == SVM_ON) {
		//la fonction SvmStd fournit directement des rapports cycliques compris entre 0 et 1.
		SVMSector = GMCLIB_SvmStd_FLT(&PWM_abc_Out,&AlphaBetaOut);
	}
	else {
		//ClarkInv fournit un rsultat compris entre -1 et +1.
		//Il faut donc le ramener entre 0 et 1 pour tre compatible avec les valeurs de rapport
		//cycliques acceptables
		GMCLIB_ClarkInv_FLT(&PWM_abc_Out,&AlphaBetaOut);
		PWM_abc_Out.fltArg1 = MLIB_Add_FLT(PWM_abc_Out.fltArg1,1);
		PWM_abc_Out.fltArg2 = MLIB_Add_FLT(PWM_abc_Out.fltArg2,1);
		PWM_abc_Out.fltArg3 = MLIB_Add_FLT(PWM_abc_Out.fltArg3,1);
		PWM_abc_Out.fltArg1 = MLIB_Div_FLT(PWM_abc_Out.fltArg1,2);
		PWM_abc_Out.fltArg2 = MLIB_Div_FLT(PWM_abc_Out.fltArg2,2);
		PWM_abc_Out.fltArg3 = MLIB_Div_FLT(PWM_abc_Out.fltArg3,2);
	}

	//Uabc_commande n'est utile qu' des fins de debug (et pour l'utilisation du modle de moteur virtuel).
	Uabc_Commande.fltArg1 = MLIB_Mul_FLT(PWM_abc_Out.fltArg1,UCC_MAX_V);
	Uabc_Commande.fltArg2 = MLIB_Mul_FLT(PWM_abc_Out.fltArg2,UCC_MAX_V);
	Uabc_Commande.fltArg3 = MLIB_Mul_FLT(PWM_abc_Out.fltArg3,UCC_MAX_V);
	Uabc_Commande.fltArg1 = MLIB_Sub_FLT(Uabc_Commande.fltArg1,UPHASE_MAX_V);
	Uabc_Commande.fltArg2 = MLIB_Sub_FLT(Uabc_Commande.fltArg2,UPHASE_MAX_V);
	Uabc_Commande.fltArg3 = MLIB_Sub_FLT(Uabc_Commande.fltArg3,UPHASE_MAX_V);


	//conversion au format F16
	Uabc_Duty_F16.f16Arg1 = MLIB_ConvertPU_F16FLT(PWM_abc_Out.fltArg1);
	Uabc_Duty_F16.f16Arg2 = MLIB_ConvertPU_F16FLT(PWM_abc_Out.fltArg2);
	Uabc_Duty_F16.f16Arg3 = MLIB_ConvertPU_F16FLT(PWM_abc_Out.fltArg3);

	return(Uabc_Duty_F16);
}


/*****************************************************************************/
/* User_Speed_Ramp                                                  */
/* Description: cette fonction calcule le prochain point de vitesse de la rampe dfinissant
 * la consigne de vitesse pour le moteur. LA variable d'entre Previous_Speed_RPM dfinit la
 * vitesse en cours (en RPM). Speed_Consigne_RPM dfinit  * la vitesse finale  atteindre
 * (en RPM). delta_Speed dfinit la pente de la rampe de vitesse (en RPM/appel de cette fonction).
 * La fonction renvoie la valeur de la future vitesse de consigne. */
/*****************************************************************************/
tFloat User_Speed_Ramp(tFloat Previous_Speed_RPM,tFloat Speed_Consigne_RPM,tFloat delta_Speed_RPM) {
	tFloat Speed_Output;

	//Tant que la commande de vitesse n'est pas gale  la consigne, on cherche  l'augmenter
	//(ou la diminuer) avec une acclration donne par ACCELERATION_RAMPE_VITESSE (en RPM).

	if (Previous_Speed_RPM <= Speed_Consigne_RPM) { //on doit acclrer
		if ((MLIB_Sub_FLT(Speed_Consigne_RPM ,Previous_Speed_RPM)) < delta_Speed_RPM) {
			Speed_Output = Speed_Consigne_RPM;
		}
		else {
			Speed_Output = MLIB_Add_FLT(Previous_Speed_RPM,delta_Speed_RPM);
		}
	}
	else if (Previous_Speed_RPM > Speed_Consigne_RPM) { //on doit dclrer
		if ((MLIB_Sub_FLT(Previous_Speed_RPM,Speed_Consigne_RPM)) < delta_Speed_RPM) {
			Speed_Output = Speed_Consigne_RPM;
		}
		else {
			Speed_Output = MLIB_Sub_FLT(Previous_Speed_RPM,delta_Speed_RPM);
		}
	}
	return(Speed_Output);
}

/*****************************************************************************/
/* Exec_Slow_Loop                                                  */
/* Description: cette fonction gre l'excution de la boucle dite Slow Loop.
 * Cette boucle gre la mise  gre de la commande de vitesse, dtermine  partir de la
 * consigne de vitesse, puis calcule les rfrences de tension Ud et Uq  appliquer
 * au moteur.
 * Les tensions de phase (Ud,Uq) reprsentent des tensions exprimes en V.  */
/*****************************************************************************/
SWLIBS_2Syst_FLT Exec_Slow_Loop(void) {
	SWLIBS_2Syst_FLT Udq_calc;

	Udq_calc.fltArg1 = 0; //le flux est fix  0 !
	Udq_calc.fltArg2 = 0;

	//mise  jour de la commande de vitesse  partir de la consigne de vitesse courante.
	Commande_Vitesse = User_Speed_Ramp(Commande_Vitesse,Consigne_Vitesse,D_Vitesse);

	//calcul de la commande en tension Uq,  partir d'une look-up table
	Udq_calc.fltArg2 = U_F_LUT(Commande_Vitesse);

	return(Udq_calc);
}

/*****************************************************************************/
/* U_F_LUT                                                  */
/* Description: cette fonction dtermine la tension Uq (en V)  appliquer au moteur pour
 * une commande de vitesse donne (Vitesse_in en RPM). S'apparente  une LUT.
 */
/*****************************************************************************/
tFloat U_F_LUT(tFloat Vitesse_in) {
	tFloat Uq_ref = 0;

	if ((Vitesse_in <= 1500) & (Vitesse_in >= -1500)) {
		if ((Vitesse_in > 500) | (Vitesse_in < -500)) {
			Uq_ref = Vitesse_in*0.006;
		}
		else if (Vitesse_in > 0) {
			Uq_ref = 3;
		}
		else if (Vitesse_in < 0) {
			Uq_ref = -3;
		}
		else {
			Uq_ref = 0;
		}
	}
	else if (Vitesse_in > 1500) {
		Uq_ref = 9;
	}
	else if (Vitesse_in < -1500) {
		Uq_ref = -9;
	}

	return(Uq_ref);
}


/*****************************************************************************/
/*Estimate_Rotor_Angle                                                       */
/* Description: cette fonction dtermine l'angle du rotor  partir de la vitesse du rotor
 * exprime en RPM (Vitesse_RPM) et de la valeur prcdente de l'angle rotor Angle_Rotor exprim en rad.
 * Comme la commande est de type U/f, on ne lit pas l'angle  partir du rsolveur.
 * L'angle est compris entre [-pi et +pi] et exprim en radians.                                        */
/*****************************************************************************/
tFloat Estimate_Rotor_Angle(tFloat Vitesse_RPM, tFloat Angle_rotor) {
	tFloat Angle_incr_urad, Angle_rotor_init_urad, Angle_rotor_fin_urad; //angles exprims en radian.
	tFloat Angle_rotor_fin_rad; //exprim en radian

	//Angle  (T0+dt) = Angle (T0) + vitesse_RPM * pi/30 * PP * TFST_LOOP, o PP est le
	//nb de paires de ples, T_FST_LOOP = dt = priode de la boucle Fast Loop. pi/30 = 0.1047.
	//on calcule d'abord l'incrment d'angle rotor produit par la vitesse du rotor pendant la dure T_FAST_LOOP
	Angle_incr_urad = MLIB_Mul_FLT(const_calcul_RPM_Radian,Vitesse_RPM); //comme T_FAST_LOOP_US est exprim
		//en s, Angl_incr est exprime en rad.

	Angle_rotor_init_urad = MLIB_Mul_FLT(Angle_rotor,1000000);
	Angle_rotor_fin_urad = MLIB_Add_FLT(Angle_incr_urad,Angle_rotor_init_urad);

	if (Angle_rotor_fin_urad > PI_MICRORAD) {
		Angle_rotor_fin_urad = MLIB_Sub_FLT(Angle_rotor_fin_urad,TWO_PI_MICRORAD);
	}
	else if (Angle_rotor_fin_urad < -PI_MICRORAD) {
		Angle_rotor_fin_urad = MLIB_Add_FLT(Angle_rotor_fin_urad,TWO_PI_MICRORAD);
	}
	Angle_rotor_fin_rad = MLIB_Div_FLT(Angle_rotor_fin_urad,1000000);

	return(Angle_rotor_fin_rad);
}

/*****************************************************************************/
/* Convert_CmdDuty_F16_INT16                                                 */
/* Description: la fonction convertit les rapports cycliques des 3 commandes PWM
 * dfinies au format F16 (compris entre -1 et + 1, en des rapports cycliques au
 *  format int16, compatibles avec les registres du priph FlexPWM.
 * Leurs valeurs sont comprises entre 0 et PeriodPWM (= 1000). Les rsultats de
 * sortie sont les variables globales Cmd_DutyA, Cmd_DutyB et Cmd_DutyC. */
/*****************************************************************************/
void Convert_CmdDuty_F16_INT16(SWLIBS_3Syst_F16 Cmd_Duty_F16) {
	uint32_t cmd_dutyA_32, cmd_dutyB_32, cmd_dutyC_32;

	//mise  jour des rapports cycliques envoys au FlexPWM. Il faut d'abord passer du format Q1.15 au
	//format uint16_t et mettre le chiffre entre 0 et 1000.
	cmd_dutyA_32 = PWMPeriod*Cmd_Duty_F16.f16Arg1;
	cmd_dutyB_32 = PWMPeriod*Cmd_Duty_F16.f16Arg2;
	cmd_dutyC_32 = PWMPeriod*Cmd_Duty_F16.f16Arg3;

	cmd_dutyA_32 = cmd_dutyA_32/0x7FFF;
	cmd_dutyB_32 = cmd_dutyB_32/0x7FFF;
	cmd_dutyC_32 = cmd_dutyC_32/0x7FFF;

	Cmd_DutyA = (uint16_t) (cmd_dutyA_32);
	Cmd_DutyB = (uint16_t) (cmd_dutyB_32);
	Cmd_DutyC = (uint16_t) (cmd_dutyC_32);
}



/*****************************************************************************/
/* ADC0_EOC_FAST_LOOP_isr                                                 */
/* Description: gestion de l'interruption lie  une nouvelle acquisition de IphA, qui cadence
 * l'excution de la boucle Fast Loop. Dans cette boucle, l'acquisition des courants et
 * la mise  jour des rapports cycliques pour la commance sont effectues.
 * On lance aussi la boucle dite Slow_loop de mise  jour de la vitesse de consigne
 * cptr_slow_loop < nb_exec_FL_max_Exec_SL.*/
/*****************************************************************************/
void ADC0_EOC_FAST_LOOP_isr(void) {
	SWLIBS_3Syst_F16 Cmd_Duty_F16;
	uint16_t angle_read;
	tFloat d_angle;
	uint8_t PhaseCurrentRead_OK;

	Set_pad_out(User_Test,OUT_ON);

	//FONCTIONNEMENT NORMAL : excution normal du pilotage du moteur (excution normale des slow et fast loop).
	if (Mode_Sequence_Moteur == MODE_FCT_NORM) {

		//angle rotor et courants de phase obtenus par mesure
		if (Mode_Lecture_Courant_Angle == MOTEUR_REEL) {
			//rcupration des courants de phase par la lecture des tensions aux bornes des rsistances de shunt
			//pour l'instant, la fonction ignore le cas d'une lecture errone des courants de phase.
			PhaseCurrentRead_OK = Get_ThreePhase_Current_MotorGD_FLT();

			//rcupration de l'angle rotor
			//lecture de l'angle "mcanique" = position du rotor. Si on excute la boucle slow loop,
			//on positionne la prochaine adresse  lire sur DEV_STAT6 (lecture de vitesse). Sinon,
			//sur DEV_STAT5 (lecture de la position)
			angle_read = pga411_ReadReg_CRC_connu_OneRead(DEV_STAT5,CRC_READ_REG_DEV_STAT5);

			//conversion de l'angle mcanique en angle "lectrique", exprim en radian compris entre -pi et +pi
			//pour tre compatible avec les fonctions d'AMMCLIB
			angle_rotor_elec = Convert_AngleElec_Rad_FLT(angle_read,Resolution_PGA411,PAIRE_POLE);
			//il ne faut pas diviser par 3 l'angle rotor lectrique pour obtenir l'angle rotor mcanique.
			//Cela a pour consquence de diviser par 3 la valeur de la vitesse mcanique.
			angle_rotor_meca = Convert_AngleMeca_Rad_FLT(angle_read,Resolution_PGA411);
		 }
		//angle rotor et courants de phase obtenus  l'aide d'un modle de moteur virtuel (pour des essais de validation
		//du code uniquement)
		else {
			Get_Phase_Current_Virtuel();
			Get_Angle_Rotor_Virtuel();
		}

		//excution boucle fast loop
		Cmd_Duty_F16 = Exec_Fast_Loop(Udq_Commande);


		//conversion des rapports cycliques calculs par la fonction Exec_Fast_Loop (type F16) --> format
		//compatible avec le priphrique FlexPWM.
		Convert_CmdDuty_F16_INT16(Cmd_Duty_F16);
		//mise  jour des rapports cycliques des 3 commandes PWM
		gd3000_Update_DutyCycle_Cmd_PWM_ABC(Cmd_DutyA,Cmd_DutyB,Cmd_DutyC,PWMPeriod,DeadTime_PWM,Align_PWM);

		//xcution  de la boucle slow loop au bout de nb_exec_FL_max_Exec_SL excutions de la Fast Loop pour
		// mettre  jour la commmande Udq et mise  jour de la commande de vitesse.
		//Entre temps, on acquiert la variation d'angle rotor pour dduire la vitesse du rotor.
		if (cptr_slow_loop < nb_exec_FL_max_Exec_SL) {

			//intgration de l'angle rotor mcanique parcouru depuis la dernire excution de la slow loop :
			if (cptr_slow_loop == 0) {
				angle_rotor_meca_precedent = angle_rotor_meca;
			}
			else {
				//ATTENTION AU PASSAGE 360 --> 0 ou l inverse !!
				d_angle = MLIB_Sub_FLT(angle_rotor_meca,angle_rotor_meca_precedent);

				if (d_angle < (-d_angle_rad_FST_LOOP_max)) {
					//d_angle = 360 + angle-angle_precedent
					d_angle = MLIB_Add_FLT((tFloat) (TWO_PI_FLT),d_angle);
				}
				else if (d_angle > d_angle_rad_FST_LOOP_max) {
					//d_angle = angle-angle_precedent-360
					d_angle = MLIB_Sub_FLT(d_angle,(tFloat) (TWO_PI_FLT));
				}

				increment_angle_rotor = MLIB_Add_FLT(increment_angle_rotor,d_angle);
				angle_rotor_meca_precedent = angle_rotor_meca;
			}

			cptr_slow_loop++;
		}
		else {
			cptr_slow_loop = 0;

			velocity_rotor_meca = MLIB_Div_FLT(increment_angle_rotor,tps_integration_vitesse); //vitesse en rad/us
			velocity_rotor_meca = MLIB_Mul_FLT(velocity_rotor_meca,cte_calcul_RPM_from_rad_us); //conversion en RPM
			increment_angle_rotor = (tFloat) (0);

			Udq_Commande = Exec_Slow_Loop();
		}
	}

	//MODE CALIBRATION INITIALE : on compense les offsets des mesures de courant de phase. On effectue un nb
	//NB_MAX_MES_CAL de mesures pour dterminer l'offset en entre des points de mesure.
	else if (Mode_Sequence_Moteur == MODE_FCT_CAL) {
		Perform_Current_Meas_Calibration();
		if (Cal_Current_Done == CAL_DONE) {
			Mode_Sequence_Moteur = MODE_FCT_NORM; //passage en fonctionnement normal
		}
	}

	//RAZ du flag ADC0
	ADC_Clear_All_Flags(ADC0);
	Set_pad_out(User_Test,OUT_OFF);
}
